#!/bin/sh

set -x

# This script will setup for a manual patch or delayed patch fix


export PATH=/control4/bin:/usr/sbin:/bin:/usr/bin:/sbin
export LD_LIBRARY_PATH=/control4/lib:/opt/lib


SHT_NAME="C4_252_AMS_Multi_Stream_Patch";
GET_FILE="http://update.control4.com/patches/${SHT_NAME}";
PVER="283237";
SCRIPT_VERSION="2.5.3.00020"

CHK_FILE="/etc/c4i/TuneIn.c4i";

ST_NAME="fix-ams-${PVER}.sh"
P_SCRIPT="patchscript-${PVER}"
MOD_NAME="pdl-${PVER}.tgz"    


ETC="/etc";
C4BIN="/control4/bin";
TMPDIR="/tmp";
MNTINT="/mnt/internal";
LOGDIR="/var/log";

EMAIL_STATUS="${LOGDIR}/emailmessage.txt";
PALOG="${LOGDIR}/patch_applied";
PELOG="${LOGDIR}/patch_error";
CRONFILE="${ETC}/crontabs/root";

DPKGL="dpkg -l";
UNTAR="tar -xzf ";
KILLALL="killall -9";
MD5="md5sum"; 
CHMOD="chmod +x ";
WGET="wget ";

GVERSION=0
NeedToDownload=0;

for i in $*; do
    arg=`echo ${i} | cut -d'=' -f1 | sed 's/-//g'`
    value=`echo ${i} | cut -d'=' -f2`
    eval export ${arg}=${value}
done



UpdateMessage()
{
    # $1 is the message string to update the Email File
    # $2 is the message file to write

    echo -e "-- $1 --" | tee -a $2    
}

AllPatched()
{
    UpdateMessage "Completed: $1"
    echo -e "Script: ${ST_NAME} Version: ${SCRIPT_VERSION}\n\n" > ${PALOG}
    echo -e "Completed: $1" >> ${PALOG}
        
    if [ ${GVERSION} -eq 0 ] ; then
        RemoveCrontab
    fi
    
    CleanOnExit 2
}

die()
{
    UpdateMessage "Error: ${1} " >> ${PELOG}
    # clean up and exit gracefully
    
    RemoveCrontab
    CleanOnExit 1
}


RemoveOldPatchFiles()
{
    rm -f ${EMAIL_STATUS} ${PALOG} ${PELOG} &
}

CleanOnExit()
{
    # $1 is for the exit code

    rm -f ${LOGDIR}/${ST_NAME} ${LOGDIR}/${P_SCRIPT}* ${LOGDIR}/${MOD_NAME}* ${LOGDIR}/*tgz* 
    
    #DoCheckIn &
    #sleep 4   # Enough time to get some data from DoCheckIn into status email before sending
    
    exit ${1}
}

CheckTheVersion()
{

   if [ ${RH_Major} -eq 2 ] && [ ${RH_Minor} -eq 5 ] && [ ${RH_Patch} -eq 2 ] ; then
       UpdateMessage "This OS is the correct version for the ${SHT_NAME} Program."  ${EMAIL_STATUS}
       GVERSION=1
   else
       die "The Major.Minor.Rev version # is not a valid version number to run the ${SHT_NAME} Program."
   fi
}

IsPatchApplied()
{

    CHk_FILE_MD5="44475f87fcb773498e007568e4a6490b";
    
    # Let's see if we need to install Patch
    # Let's see if crontab is already setup
    if [ -e ${CHK_FILE} ]; then

        CHK_NAME_MD5=`${MD5} ${CHK_FILE} | cut -c 1-32`

        if [ "${CHk_FILE_MD5}"x == "${CHK_NAME_MD5}"x ] ; then
            AllPatched    
        fi
    fi
    

}


CheckAndSetupCron()
{
   UpdateMessage "Updating the ${CRONFILE} "  ${EMAIL_STATUS}
   cp ${CRONFILE} ${CRONFILE}.${PVER}
   echo "31 3  *   *   *  ${MNTINT}/${ST_NAME} dn=0" >> ${CRONFILE}
   ${KILLALL} crond
}

CleanMNT()
{
    UpdateMessage "Removing all the script files from ${MNTINT} "  ${EMAIL_STATUS}
    rm -f ${MNTINT}/${ST_NAME} ${MNTINT}/patchscr* ${MNTINT}/pdl* ${MNTINT}/*tgz* ${CRONFILE}.* &
}

RemoveCrontab()
{
    UpdateMessage "Restoring the original ${CRONFILE} "  ${EMAIL_STATUS}
    
    cat ${CRONFILE} | grep -v "${ST_NAME}" > ${CRONFILE}.${PVER}
    RESTORE=`cat ${CRONFILE}.${PVER}` 
    if [ -n "${RESTORE}" ] ; then
        UpdateMessage  "Cleaning up the Cron File and removing ${ST_NAME}."  ${EMAIL_STATUS}    
        mv ${CRONFILE}.${PVER} ${CRONFILE}
    else
        UpdateMessage  "Error: The Temp Cron File was completely empty on ${ST_NAME}."  ${EMAIL_STATUS}    
    fi
    
    ${KILLALL} crond
    
    CleanMNT
}

DoCheckIn () 
{

    C4Soap='<c4soap name="SendToDevice" session="0" operation="RWX" category="composer" async="False"><param name="iddevice" type="number">100117</param><param name="data" type="string"><devicecommand owneridtype="" owneriditem="-1"><command>CheckIn</command><params></params></devicecommand></param></c4soap>'

#
# The controller has to check in after it drops off the vpn to get
# it to stop calling vpnclient.asmx.  Check-ins while it is still connected to
# the vpn don't seem to do anything.
#
# Watch the tap adapter status with ifconfig.  When tap adapter is gone, do a check in.
# Exit in any case after an hour since openvpn re-authentications happen hourly.  If
# controller is still connected after an hour, it probably has 4sight and will remain
# connected and won't be a problem with excessive vpnclient calls.
#

    SleepInterval=60

    MinutesLeft=65     # An hour plus a margin.
    while [ $MinutesLeft -gt 0 ]; do
        ifconfig tap0 > /dev/null 2>&1
        if [ $? -eq 0 ]; then
            MinutesLeft=$((${MinutesLeft} - 1));
            sleep $SleepInterval
        else
            
            ${KILLALL} openvpn
            sleep 1
            echo "$C4Soap" |  tr '\012' '\000' | telnet localhost 5020
            exit 0
        fi
    done
    exit 0
}

CheckFileDownloaded()
{
    if [ ! -e ${LOGDIR}/${1} ] && [ ! -e ${MNTINT}/${1} ]; then
        NeedToDownload=1;
        return;
    fi
   
    if [ -e ${LOGDIR}/${1} ] && [ ! -e ${MNTINT}/${1} ] ; then
        cp ${LOGDIR}/${1} ${MNTINT}/${1}    
        cp ${LOGDIR}/${1}.${MD5} ${MNTINT}/${1}.${MD5}    
    
    elif [ ! -e ${LOGDIR}/${1} ] && [ -e ${MNTINT}/${1} ] ; then
        cp ${MNTINT}/${1} ${LOGDIR}/${1}     
        cp ${MNTINT}/${1}.${MD5} ${LOGDIR}/${1}.${MD5}    
    fi
}

SetupCronFirst()   
{   
   UpdateMessage "Setting up the Crontab file to delay the execution of the script in ${MNTINT}/${ST_NAME}."  ${EMAIL_STATUS}
   if [ ! -e "${MNTINT}/${ST_NAME}" ] ; then 
           cp ${LOGDIR}/${ST_NAME} ${MNTINT}/${ST_NAME}
   fi
       
   IsInstalled=`cat ${CRONFILE} | grep -i ${MNTINT}/${ST_NAME}`
   
   if [ -z "${IsInstalled}" ] ; then
           
       #############################################################
       #
       # We need to see if the fixit script is setup in the crontab
       #
       #############################################################
       
       UpdateMessage "Adding a setting in the Crontab file."  ${EMAIL_STATUS}
       CheckAndSetupCron
       
   fi
}        

DownloadAndUnTar()
{   

   # $1 is the file name
   # $2 is the location of where to untar the files
    
    cd ${LOGDIR}
   
    CheckFileDownloaded ${1}
    
    if [ ${NeedToDownload} -eq 1 ] ; then   
        UpdateMessage "Downloading the following file ${WGET} ${GET_FILE}/${1} "  ${EMAIL_STATUS}
        ${WGET} ${GET_FILE}/${1}
        ${WGET} ${GET_FILE}/${1}.${MD5}
    fi
    
    MOD_NAME_MD5=`${MD5} ${1} | cut -c 1-32`
    RECORDED_MD5=`cat ${1}.${MD5} | cut -c 1-32`

    [ -s ${1} ] || die "${1} not found"

    [ -s ${1}.${MD5} ] || die "${1}.${MD5} not found"

    [ -z "${MOD_NAME_MD5}" ] && die "Could not get the ${MD5} from the downloaded file ${1}"
   
    [ -z "${RECORDED_MD5}" ] && die "Could not get the ${MD5} from the downloaded file ${1}.${MD5}"

    [ "${RECORDED_MD5}"x == "${MOD_NAME_MD5}"x ] || die "The file ${1} downloaded ${MD5} value didn't match the recorded ${MD5} value for the downloaded file ${MOD_NAME_MD5} and Recorded ${MD5} ${RECORDED_MD5}"

    
    #############################################################
    #
    #   We have the files on the Master Controller so let's not download again.
    #   We need to save them on the /mnt/internal for later.
    #
    #############################################################
   
    if [ ${NeedToDownload} -eq 1 ] ; then   
        CheckFileDownloaded ${1}
    fi    

    #############################################################
    #
    # Untarring the downloaded file
    #
    #############################################################
   
    UpdateMessage "UnTarring the downloaded file ${1}"  ${EMAIL_STATUS}
    ${UNTAR} ${1} -C $2
}

StartUp()
{

    # We need to check the uptime of the system
    MainUptime=`uptime | grep -i days | cut -d ',' -f1 | cut -d ' ' -f4`
    
    if [ -z "${MainUptime}" ] ; then
    
       MainUptime=0;
    
    fi

    # Get the version of the Controller
    Get_Cntl_Ver=`${DPKGL} | grep system-version | tr -s ' '| cut -d ' ' -f3`
    RH_Major=`echo ${Get_Cntl_Ver} | tr -s ' '| cut -d ' ' -f3 | cut -d '.' -f1`
    RH_Minor=`echo ${Get_Cntl_Ver} | tr -s ' '| cut -d ' ' -f3 | cut -d '.' -f2`
    RH_Patch=`echo ${Get_Cntl_Ver} | tr -s ' '| cut -d ' ' -f3 | cut -d '.' -f3`

    # Get the Host Processor Type
    DEVICE_TYPE=`uname -m`;

    # Get the Host Controller Name
    CN_NAME=`uname -a | cut -d' ' -f2`
    DEVICE_NAME=`uname -v | cut -d' ' -f1 | cut -d '-' -f2 | cut -d '.' -f1`
    #DEVICE_MAC=`ifconfig | grep -i 'eth0' | cut -d 'W' -f2 | cut -d' ' -f2 `
    DEVICE_MAC=`cat /etc/smb.conf | grep -i netbios | cut -d '=' -f2 | cut -d '-' -f1 | tr -s ' '`

    [ -z "${dn}" ] && dn=1
    [ -n "${cn}" ] && CN_NAME=${cn}

    UpdateMessage "Common Name = ${CN_NAME}"  ${EMAIL_STATUS}
    UpdateMessage "Home Controller Version = ${Get_Cntl_Ver}"  ${EMAIL_STATUS}
    UpdateMessage "Host MAC = ${DEVICE_MAC}"  ${EMAIL_STATUS}
    UpdateMessage "Device Type = ${DEVICE_TYPE}"  ${EMAIL_STATUS}
    UpdateMessage "Device Name = ${DEVICE_NAME}"  ${EMAIL_STATUS}
    UpdateMessage "Controller uptime in Days = ${MainUptime}"  ${EMAIL_STATUS}

}

 
#############################################################
#
#  MAIN of the Script
#
#############################################################

cd ${LOGDIR}

#############################################################
#
#  Starting the MAIN Part of the Script
#
#############################################################

StartUp 
    
#############################################################
#
# Cleaning up the old files on the system
#
#############################################################

RemoveOldPatchFiles

#############################################################
#
# Lets Check the version of the system before we do anything
#
#############################################################
CheckTheVersion

#############################################################
#
# Let's see if the patch has been installed or if already Applied
#
#############################################################
IsPatchApplied

#############################################################
#
# We need to download the files from the update server
# If the files have not already been downloaded
#
#############################################################

DownloadAndUnTar ${MOD_NAME} ${LOGDIR}

#############################################################
#
# Delaying the patch install until later
#
#############################################################

if [ ${dn} -eq 1 ] ; then
    
    #############################################################
    #
    # Lets Check the version of the system before we do anything
    #
    #############################################################
    
    SetupCronFirst
    
    UpdateMessage "Delaying the deployment of the patch on the system until 3:30 AM."  ${EMAIL_STATUS}
    CleanOnExit 1;

else
    UpdateMessage "Manually Deploying the patch to the system Now."  ${EMAIL_STATUS}
    ${CHMOD} ${LOGDIR}/${P_SCRIPT}* 
    ${LOGDIR}/${P_SCRIPT}
    
    #############################################################
    #
    # The Download files are pushed to mnt/internal and 
    # we need to clean up crontab and these files.
    #
    #############################################################
    RemoveCrontab
    CleanMNT
fi
   

##########################################################        
# Sending the update information to control4             #
##########################################################

CleanOnExit 0;

